#Example of calling MG (Managing Gigabytes) mgquery from a Python 1.5.2 Application
#running under NT 4.0.

import sys
import os
import string

def prepForBooleanDocnumsQuery(mgDir, userName, query):
	#use for the initial boolean query
	f = open(mgDir + userName + "\\.mgrc", "w")
	f.write(".set mode docnums\n" +
			".set query boolean\n")
	f.close()
	return

def prepForDocnumsHeadsQuery(mgDir, userName, query):
	#use to get the document header (1st line) for the docnum'th document
	f = open(mgDir + userName + "\\.mgrc", "w")
	f.write(".set mode heads\n" +
			".set query docnums\n")
	f.close()
	return

def prepForDocnumsTextQuery(mgDir, userName, query):
	#use to get the full text for the docnum'th document
	f = open(mgDir + userName  + "\\.mgrc", "w")
	f.write(".set mode text\n" +
			".set query docnums\n")
	f.close()
	return

def readQueryResults(mgDir, userName):
	#read the results of the last query into a list and return
	f = open(mgDir + userName + "\\qresults.txt")
	lines = f.readlines()
	f.close()
	return lines

def invokeMgQuery(mgDir, userName, docCollectionName):
	#invoke mgquery
	#
	#must use "<" and ">" instead of ".input" and ".output" commands
	#which cause a Dr. Watson dialog on my system.
	#
	os.system(mgDir + userName + "\\mgquery" +
			  " -d " + mgDir + userName + "\\" +
			  " -f " + docCollectionName +
			  " < " + mgDir + userName + "\\query.txt" +
			  " > " +  mgDir + userName + "\\qresults.txt")
	return

def runDocnumsQuery(mgDir, userName, docCollectionName, query):
	prepForBooleanDocnumsQuery(mgDir, userName, query)
	invokeMgQuery(mgDir, userName, docCollectionName)
	return

def runHeadsQuery(mgDir, userName, docCollectionName, query):
	prepForDocnumsHeadsQuery(mgDir, userName, query)
	invokeMgQuery(mgDir, userName, docCollectionName)
	return

def runTextQuery(mgDir, userName, docCollectionName, query):
	prepForDocnumsTextQuery(mgDir, userName, query)
	invokeMgQuery(mgDir, userName, docCollectionName)
	return

def writeQuery(mgDir, userName, query):
	#write the query to a file, which will take the place of STDIN
	#and keep mgquery from coming up in interactive mode.
	f = open(mgDir + userName + "\\query.txt", "w")
	f.write(query + "\n")
	f.close()
	return

def getDocNumFromHeads(heading):
	#get the document number, which is the first "word" of the heading info.
	#that is returned by the "heads mode" query.
	words = string.split(heading, " ")
	return words[0]

def main():
	if len(sys.argv) < 5:
		print
		print "This program is an example of how to use the Managing Gigabytes"
		print "mgquery.exe binary from a Python application. It was tested on NT 4.0 using"
		print "Python 1.5.2."
		print
		print "USAGE:"
		print "python mgQueryFromPython.py <mgDir> <userName> <docCollectionName> <query>"
		print
		print "EXAMPLE:"
		print "python mgQueryFromPython.py c:\mg\ mconley alice flamingo"
		print
		print "Notes:"
		print "Program expects a folder named <userName> inside the <mgDir> folder."
		print "The files for the collection whose base name is <docCollectionName> should"
		print "be inside the <userName> folder. The mgquery binary (mgquery.exe) should"
		print "also be inside the <userName> folder."
		return

	mgDir = sys.argv[1]
	userName = sys.argv[2]
	tempFile = mgDir + userName + "\\allTextFromQuery.txt"
	try:
		os.remove(tempFile)
	except:
		pass
	docCollectionName = sys.argv[3]
	originalQuery = query = sys.argv[4]
	os.chdir(mgDir + userName)

	#run boolean query, getting document numbers for the result
	writeQuery(mgDir, userName, query)
	runDocnumsQuery(mgDir, userName, docCollectionName, query)
	results = readQueryResults(mgDir, userName)
	results = map(string.strip, results)

	#for each document number, get a heading (1st line of document). This info.
	#might be used in an html doc showing the search results, for instance.
	resultsForHtml = []
	for r in results[1:]:
		query = r
		writeQuery(mgDir, userName, query)
		runHeadsQuery(mgDir, userName, docCollectionName, query)
		headResults = readQueryResults(mgDir, userName)
		resultsForHtml.append(string.strip(headResults[0]))

	#for each document number, get the text and append it to a temporary file for viewing.
	#Could be returned in response to an http request, for instance.
	f = open(tempFile, "a")
	f.write("Results of query " + `originalQuery` + "\n\n")
	f.close()
	for rh in resultsForHtml:
		query = getDocNumFromHeads(rh)
		writeQuery(mgDir, userName, query)
		runTextQuery(mgDir, userName, docCollectionName, query)
		text = readQueryResults(mgDir, userName)
		f = open(tempFile, "a")
		for line in text:
			f.write(line)
		f.close()
	return

if __name__ == "__main__":
	main()
