#Remove all HTML from a page, leaving only the text, which can be stored in the ZODB, indexed, etc.

import re
import string
import sys
import Porter

#use stopWordsMerger.py utility to re-create the following dictionary, if necessary
stopwords = {
	"A" : None,
	"A'S" : None,
	"ABLE" : None,
	"ABOUT" : None,
	"ABOVE" : None,
	"ACCORDING" : None,
	"ACCORDINGLY" : None,
	"ACROSS" : None,
	"ACTUALLY" : None,
	"AFTER" : None,
	"AFTERWARDS" : None,
	"AGAIN" : None,
	"AGAINST" : None,
	"AIN'T" : None,
	"ALL" : None,
	"ALLOW" : None,
	"ALLOWS" : None,
	"ALMOST" : None,
	"ALONE" : None,
	"ALONG" : None,
	"ALREADY" : None,
	"ALSO" : None,
	"ALTHOUGH" : None,
	"ALWAYS" : None,
	"AM" : None,
	"AMONG" : None,
	"AMONGST" : None,
	"AMOUNGST" : None,
	"AMOUNT" : None,
	"AN" : None,
	"AND" : None,
	"ANOTHER" : None,
	"ANY" : None,
	"ANYBODY" : None,
	"ANYHOW" : None,
	"ANYONE" : None,
	"ANYTHING" : None,
	"ANYWAY" : None,
	"ANYWAYS" : None,
	"ANYWHERE" : None,
	"APART" : None,
	"APPEAR" : None,
	"APPRECIATE" : None,
	"APPROPRIATE" : None,
	"ARE" : None,
	"AREN'T" : None,
	"AROUND" : None,
	"AS" : None,
	"ASIDE" : None,
	"ASK" : None,
	"ASKING" : None,
	"ASSOCIATED" : None,
	"AT" : None,
	"AVAILABLE" : None,
	"AWAY" : None,
	"AWFULLY" : None,
	"B" : None,
	"BACK" : None,
	"BE" : None,
	"BECAME" : None,
	"BECAUSE" : None,
	"BECOME" : None,
	"BECOMES" : None,
	"BECOMING" : None,
	"BEEN" : None,
	"BEFORE" : None,
	"BEFOREHAND" : None,
	"BEHIND" : None,
	"BEING" : None,
	"BELIEVE" : None,
	"BELOW" : None,
	"BESIDE" : None,
	"BESIDES" : None,
	"BEST" : None,
	"BETTER" : None,
	"BETWEEN" : None,
	"BEYOND" : None,
	"BILL" : None,
	"BOTH" : None,
	"BOTTOM" : None,
	"BRIEF" : None,
	"BUT" : None,
	"BY" : None,
	"C" : None,
	"C'MON" : None,
	"C'S" : None,
	"CALL" : None,
	"CAME" : None,
	"CAN" : None,
	"CAN'T" : None,
	"CANNOT" : None,
	"CANT" : None,
	"CAUSE" : None,
	"CAUSES" : None,
	"CERTAIN" : None,
	"CERTAINLY" : None,
	"CHANGES" : None,
	"CLEARLY" : None,
	"CO" : None,
	"COM" : None,
	"COME" : None,
	"COMES" : None,
	"COMPUTER" : None,
	"CON" : None,
	"CONCERNING" : None,
	"CONSEQUENTLY" : None,
	"CONSIDER" : None,
	"CONSIDERING" : None,
	"CONTAIN" : None,
	"CONTAINING" : None,
	"CONTAINS" : None,
	"CORRESPONDING" : None,
	"COULD" : None,
	"COULDN'T" : None,
	"COULDNT" : None,
	"COURSE" : None,
	"CRY" : None,
	"CURRENTLY" : None,
	"D" : None,
	"DE" : None,
	"DEFINITELY" : None,
	"DESCRIBE" : None,
	"DESCRIBED" : None,
	"DESPITE" : None,
	"DETAIL" : None,
	"DID" : None,
	"DIDN'T" : None,
	"DIFFERENT" : None,
	"DO" : None,
	"DOES" : None,
	"DOESN'T" : None,
	"DOING" : None,
	"DON'T" : None,
	"DONE" : None,
	"DOWN" : None,
	"DOWNWARDS" : None,
	"DUE" : None,
	"DURING" : None,
	"E" : None,
	"EACH" : None,
	"EDU" : None,
	"EG" : None,
	"EIGHT" : None,
	"EITHER" : None,
	"ELEVEN" : None,
	"ELSE" : None,
	"ELSEWHERE" : None,
	"EMPTY" : None,
	"ENOUGH" : None,
	"ENTIRELY" : None,
	"ESPECIALLY" : None,
	"ET" : None,
	"ETC" : None,
	"EVEN" : None,
	"EVER" : None,
	"EVERY" : None,
	"EVERYBODY" : None,
	"EVERYONE" : None,
	"EVERYTHING" : None,
	"EVERYWHERE" : None,
	"EX" : None,
	"EXACTLY" : None,
	"EXAMPLE" : None,
	"EXCEPT" : None,
	"F" : None,
	"FAR" : None,
	"FEW" : None,
	"FIFTEEN" : None,
	"FIFTH" : None,
	"FIFY" : None,
	"FILL" : None,
	"FIND" : None,
	"FIRE" : None,
	"FIRST" : None,
	"FIVE" : None,
	"FOLLOWED" : None,
	"FOLLOWING" : None,
	"FOLLOWS" : None,
	"FOR" : None,
	"FORMER" : None,
	"FORMERLY" : None,
	"FORTH" : None,
	"FORTY" : None,
	"FOUND" : None,
	"FOUR" : None,
	"FROM" : None,
	"FRONT" : None,
	"FULL" : None,
	"FURTHER" : None,
	"FURTHERMORE" : None,
	"G" : None,
	"GET" : None,
	"GETS" : None,
	"GETTING" : None,
	"GIVE" : None,
	"GIVEN" : None,
	"GIVES" : None,
	"GO" : None,
	"GOES" : None,
	"GOING" : None,
	"GONE" : None,
	"GOT" : None,
	"GOTTEN" : None,
	"GREETINGS" : None,
	"H" : None,
	"HAD" : None,
	"HADN'T" : None,
	"HAPPENS" : None,
	"HARDLY" : None,
	"HAS" : None,
	"HASN'T" : None,
	"HASNT" : None,
	"HAVE" : None,
	"HAVEN'T" : None,
	"HAVING" : None,
	"HE" : None,
	"HE'S" : None,
	"HELLO" : None,
	"HELP" : None,
	"HENCE" : None,
	"HER" : None,
	"HERE" : None,
	"HERE'S" : None,
	"HEREAFTER" : None,
	"HEREBY" : None,
	"HEREIN" : None,
	"HEREUPON" : None,
	"HERS" : None,
	"HERSELF" : None,
	"HI" : None,
	"HIM" : None,
	"HIMSELF" : None,
	"HIS" : None,
	"HITHER" : None,
	"HOPEFULLY" : None,
	"HOW" : None,
	"HOWBEIT" : None,
	"HOWEVER" : None,
	"HUNDRED" : None,
	"I" : None,
	"I'D" : None,
	"I'LL" : None,
	"I'M" : None,
	"I'VE" : None,
	"IE" : None,
	"IF" : None,
	"IGNORED" : None,
	"IMMEDIATE" : None,
	"IN" : None,
	"INASMUCH" : None,
	"INC" : None,
	"INDEED" : None,
	"INDICATE" : None,
	"INDICATED" : None,
	"INDICATES" : None,
	"INNER" : None,
	"INSOFAR" : None,
	"INSTEAD" : None,
	"INTEREST" : None,
	"INTO" : None,
	"INWARD" : None,
	"IS" : None,
	"ISN'T" : None,
	"IT" : None,
	"IT'D" : None,
	"IT'LL" : None,
	"IT'S" : None,
	"ITS" : None,
	"ITSELF" : None,
	"J" : None,
	"JUST" : None,
	"K" : None,
	"KEEP" : None,
	"KEEPS" : None,
	"KEPT" : None,
	"KNOW" : None,
	"KNOWN" : None,
	"KNOWS" : None,
	"L" : None,
	"LAST" : None,
	"LATELY" : None,
	"LATER" : None,
	"LATTER" : None,
	"LATTERLY" : None,
	"LEAST" : None,
	"LESS" : None,
	"LEST" : None,
	"LET" : None,
	"LET'S" : None,
	"LIKE" : None,
	"LIKED" : None,
	"LIKELY" : None,
	"LITTLE" : None,
	"LOOK" : None,
	"LOOKING" : None,
	"LOOKS" : None,
	"LTD" : None,
	"M" : None,
	"MADE" : None,
	"MAINLY" : None,
	"MANY" : None,
	"MAY" : None,
	"MAYBE" : None,
	"ME" : None,
	"MEAN" : None,
	"MEANWHILE" : None,
	"MERELY" : None,
	"MIGHT" : None,
	"MILL" : None,
	"MINE" : None,
	"MORE" : None,
	"MOREOVER" : None,
	"MOST" : None,
	"MOSTLY" : None,
	"MOVE" : None,
	"MUCH" : None,
	"MUST" : None,
	"MY" : None,
	"MYSELF" : None,
	"N" : None,
	"NAME" : None,
	"NAMELY" : None,
	"ND" : None,
	"NEAR" : None,
	"NEARLY" : None,
	"NECESSARY" : None,
	"NEED" : None,
	"NEEDS" : None,
	"NEITHER" : None,
	"NEVER" : None,
	"NEVERTHELESS" : None,
	"NEW" : None,
	"NEXT" : None,
	"NINE" : None,
	"NO" : None,
	"NOBODY" : None,
	"NON" : None,
	"NONE" : None,
	"NOONE" : None,
	"NOR" : None,
	"NORMALLY" : None,
	"NOT" : None,
	"NOTHING" : None,
	"NOVEL" : None,
	"NOW" : None,
	"NOWHERE" : None,
	"O" : None,
	"OBVIOUSLY" : None,
	"OF" : None,
	"OFF" : None,
	"OFTEN" : None,
	"OH" : None,
	"OK" : None,
	"OKAY" : None,
	"OLD" : None,
	"ON" : None,
	"ONCE" : None,
	"ONE" : None,
	"ONES" : None,
	"ONLY" : None,
	"ONTO" : None,
	"OR" : None,
	"OTHER" : None,
	"OTHERS" : None,
	"OTHERWISE" : None,
	"OUGHT" : None,
	"OUR" : None,
	"OURS" : None,
	"OURSELVES" : None,
	"OUT" : None,
	"OUTSIDE" : None,
	"OVER" : None,
	"OVERALL" : None,
	"OWN" : None,
	"P" : None,
	"PART" : None,
	"PARTICULAR" : None,
	"PARTICULARLY" : None,
	"PER" : None,
	"PERHAPS" : None,
	"PLACED" : None,
	"PLEASE" : None,
	"PLUS" : None,
	"POSSIBLE" : None,
	"PRESUMABLY" : None,
	"PROBABLY" : None,
	"PROVIDES" : None,
	"PUT" : None,
	"Q" : None,
	"QUE" : None,
	"QUITE" : None,
	"QV" : None,
	"R" : None,
	"RATHER" : None,
	"RD" : None,
	"RE" : None,
	"REALLY" : None,
	"REASONABLY" : None,
	"REGARDING" : None,
	"REGARDLESS" : None,
	"REGARDS" : None,
	"RELATIVELY" : None,
	"RESPECTIVELY" : None,
	"RIGHT" : None,
	"S" : None,
	"SAID" : None,
	"SAME" : None,
	"SAW" : None,
	"SAY" : None,
	"SAYING" : None,
	"SAYS" : None,
	"SECOND" : None,
	"SECONDLY" : None,
	"SEE" : None,
	"SEEING" : None,
	"SEEM" : None,
	"SEEMED" : None,
	"SEEMING" : None,
	"SEEMS" : None,
	"SEEN" : None,
	"SELF" : None,
	"SELVES" : None,
	"SENSIBLE" : None,
	"SENT" : None,
	"SERIOUS" : None,
	"SERIOUSLY" : None,
	"SEVEN" : None,
	"SEVERAL" : None,
	"SHALL" : None,
	"SHE" : None,
	"SHOULD" : None,
	"SHOULDN'T" : None,
	"SHOW" : None,
	"SIDE" : None,
	"SINCE" : None,
	"SINCERE" : None,
	"SIX" : None,
	"SIXTY" : None,
	"SO" : None,
	"SOME" : None,
	"SOMEBODY" : None,
	"SOMEHOW" : None,
	"SOMEONE" : None,
	"SOMETHING" : None,
	"SOMETIME" : None,
	"SOMETIMES" : None,
	"SOMEWHAT" : None,
	"SOMEWHERE" : None,
	"SOON" : None,
	"SORRY" : None,
	"SPECIFIED" : None,
	"SPECIFY" : None,
	"SPECIFYING" : None,
	"STILL" : None,
	"SUB" : None,
	"SUCH" : None,
	"SUP" : None,
	"SURE" : None,
	"SYSTEM" : None,
	"T" : None,
	"T'S" : None,
	"TAKE" : None,
	"TAKEN" : None,
	"TELL" : None,
	"TEN" : None,
	"TENDS" : None,
	"TH" : None,
	"THAN" : None,
	"THANK" : None,
	"THANKS" : None,
	"THANX" : None,
	"THAT" : None,
	"THAT'S" : None,
	"THATS" : None,
	"THE" : None,
	"THEIR" : None,
	"THEIRS" : None,
	"THEM" : None,
	"THEMSELVES" : None,
	"THEN" : None,
	"THENCE" : None,
	"THERE" : None,
	"THERE'S" : None,
	"THEREAFTER" : None,
	"THEREBY" : None,
	"THEREFORE" : None,
	"THEREIN" : None,
	"THERES" : None,
	"THEREUPON" : None,
	"THESE" : None,
	"THEY" : None,
	"THEY'D" : None,
	"THEY'LL" : None,
	"THEY'RE" : None,
	"THEY'VE" : None,
	"THICK" : None,
	"THIN" : None,
	"THINK" : None,
	"THIRD" : None,
	"THIS" : None,
	"THOROUGH" : None,
	"THOROUGHLY" : None,
	"THOSE" : None,
	"THOUGH" : None,
	"THREE" : None,
	"THROUGH" : None,
	"THROUGHOUT" : None,
	"THRU" : None,
	"THUS" : None,
	"TO" : None,
	"TOGETHER" : None,
	"TOO" : None,
	"TOOK" : None,
	"TOP" : None,
	"TOWARD" : None,
	"TOWARDS" : None,
	"TRIED" : None,
	"TRIES" : None,
	"TRULY" : None,
	"TRY" : None,
	"TRYING" : None,
	"TWELVE" : None,
	"TWENTY" : None,
	"TWICE" : None,
	"TWO" : None,
	"U" : None,
	"UN" : None,
	"UNDER" : None,
	"UNFORTUNATELY" : None,
	"UNLESS" : None,
	"UNLIKELY" : None,
	"UNTIL" : None,
	"UNTO" : None,
	"UP" : None,
	"UPON" : None,
	"US" : None,
	"USE" : None,
	"USED" : None,
	"USEFUL" : None,
	"USES" : None,
	"USING" : None,
	"USUALLY" : None,
	"UUCP" : None,
	"V" : None,
	"VALUE" : None,
	"VARIOUS" : None,
	"VERY" : None,
	"VIA" : None,
	"VIZ" : None,
	"VS" : None,
	"W" : None,
	"WANT" : None,
	"WANTS" : None,
	"WAS" : None,
	"WASN'T" : None,
	"WAY" : None,
	"WE" : None,
	"WE'D" : None,
	"WE'LL" : None,
	"WE'RE" : None,
	"WE'VE" : None,
	"WELCOME" : None,
	"WELL" : None,
	"WENT" : None,
	"WERE" : None,
	"WEREN'T" : None,
	"WHAT" : None,
	"WHAT'S" : None,
	"WHATEVER" : None,
	"WHEN" : None,
	"WHENCE" : None,
	"WHENEVER" : None,
	"WHERE" : None,
	"WHERE'S" : None,
	"WHEREAFTER" : None,
	"WHEREAS" : None,
	"WHEREBY" : None,
	"WHEREIN" : None,
	"WHEREUPON" : None,
	"WHEREVER" : None,
	"WHETHER" : None,
	"WHICH" : None,
	"WHILE" : None,
	"WHITHER" : None,
	"WHO" : None,
	"WHO'S" : None,
	"WHOEVER" : None,
	"WHOLE" : None,
	"WHOM" : None,
	"WHOSE" : None,
	"WHY" : None,
	"WILL" : None,
	"WILLING" : None,
	"WISH" : None,
	"WITH" : None,
	"WITHIN" : None,
	"WITHOUT" : None,
	"WON'T" : None,
	"WONDER" : None,
	"WOULD" : None,
	"WOULDN'T" : None,
	"X" : None,
	"Y" : None,
	"YES" : None,
	"YET" : None,
	"YOU" : None,
	"YOU'D" : None,
	"YOU'LL" : None,
	"YOU'RE" : None,
	"YOU'VE" : None,
	"YOUR" : None,
	"YOURS" : None,
	"YOURSELF" : None,
	"YOURSELVES" : None,
	"Z" : None,
	"ZERO" : None
	}

#The following dictionary entries tell whether there is useful "domain" text
#(for indexing) in the ATTRIBUTES of the tags. They do NOT refer to info. in
#between the start and the end tag if applicable.
tagAttributesToSave = {
"A" : 			("HREF", ".SRC"),
"ABBR" : 		("TITLE",),
"ACRONYM" : 	("TITLE",),
"ADDRESS" : 	None,
"APPLET" : 		("CODE",),
"AREA" : 		("HREF","ALT",),
"B" : 			None,
"BASE" : 		("HREF",),
"BASEFONT" : 	None,
"BDO" : 		None,
"BGSOUND" : 	("SRC",),
"BIG" : 		None,
"BLINK" : 		None,
"BLOCKQUOTE" : 	("CITE",),
"BODY" : 		("BACKGROUND",),
"BR" : 			None,
"BUTTON" : 		None,
"CAPTION" : 	None,
"CENTER" : 		None,
"CITE" : 		None,
"CODE" : 		None,
"COL" : 		None,
"COLGROUP" : 	None,
"DD" : 			None,
"DEL" : 		None,
"DIV" : 		None,
"!DOCTYPE" :	None,
"DL" : 			None,
"DT" : 			None,
"EM" : 			None,
"EMBED" : 		("SRC",),
"FIELDSET" : 	None,
"FONT" : 		None,
"FORM" : 		("ACTION",),
"FRAME" : 		("SRC",),
"FRAMESET" : 	None,
"H1" : 			None,
"H2" : 			None,
"H3" : 			None,
"H4" : 			None,
"H5" : 			None,
"H6" : 			None,
"HEAD" : 		None,
"HR" : 			None,
"HTML" : 		None,
"I" : 			None,
"IFRAME" : 		("SRC",),
"IMG" : 		("SRC","ALT","LOWSRC","DYNSRC"),
"INPUT" : 		("SRC",),
"INS" : 		None,
"KBD" : 		None,
"LABEL" : 		None,
"LAYER" : 		("SRC","BACKGROUND"),
"LEGEND" : 		None,
"LI" : 			None,
"LINK" : 		None,
"MAP" : 		None,
"MARQUEE" : 	None,
"MENU" : 		None,
"META" : 		("URL","CONTENT",),
"MULTICOL" : 	None,
"NOBR" : 		None,
"NOEMBED" : 	None,
"NOFRAMES" : 	None,
"NOLAYER" : 	None,
"NOSCRIPT" : 	None,
"OBJECT" : 		("CLASSID","DATA",),
"OL" : 			None,
"OPTGROUP" : 	("LABEL",),
"OPTION" : 		("VALUE",),
"P" : 			None,
"PLAINTEXT" : 	None,
"PRE" : 		None,
"Q" : 			None,
"S" : 			None,
"SAMP" : 		None,
"SCRIPT" : 		None,	#scripts are handled specially
"SELECT" : 		None,
"SMALL" : 		None,
"SPACER" : 		None,
"SPAN" : 		None,
"STRIKE" : 		None,
"STRONG" : 		None,
"STYLE" : 		None,	#styles are handled specially
"SUB" : 		None,
"SUP" : 		None,
"TABLE" : 		None,
"TBODY" : 		None,
"TD" : 			None,
"TEXTAREA" : 	None,
"TFOOT" : 		None,
"TH" : 			None,
"THEAD" : 		None,
"TITLE" : 		None,
"TR" : 			None,
"TT" : 			None,
"U" : 			None,
"UL" : 			None,
"WBR" : 		None,
"XMP" : 		None
}

#a dictionary for saving beginning tag attribute useful domain text for
#appending to the normal visible text
beginTagText = {}

def writeDebugInfo(msg):
	debugFile = open("c:\\debug.txt", "a+")
	debugFile.write(msg)
	debugFile.close()
	return

def eliminateReplaceFunction(match):
	#always return a string of one space/blank char
	#s = match.group("eliminate")
	#writeDebugInfo("eliminateReplaceFunction s=" + s + "\n")
	return " "

def keepReplaceFunction(match):
	#always return what was found
	s = match.group("keep")
	#writeDebugInfo("keepReplaceFunction s=" + s + "\n")
	return s

def saveBeginTagTextReplaceFunction(match):
	global beginTagText
	s = match.group("examine")
	tagWord = string.split(s, ">")
	tagWord = string.split(tagWord[0], " ")
	tagWord = string.strip(tagWord[0])
	tagWord = string.upper(tagWord[1:])

	#writeDebugInfo("tagWord=" + tagWord + "\n")

	tagWordEntry = tagAttributesToSave.get(tagWord, "NoEntry")
	if tagWordEntry == "NoEntry":
		writeDebugInfo("tagWord " + tagWord + " not in tagAttributesToSave dictionary.\n")
		#writeDebugInfo("returning s = " + `s` + "\n")
		beginTagText[s] = None
		return " "
	elif tagWordEntry == None:
		#writeDebugInfo("tagWordEntry == None\n")
		#writeDebugInfo("returning s = ' '\n")
		return " "
	else:
		#to do: when this part settles down, rewrite using better variable names!

		#split text based on = signs
		a = string.split(s, "=")
		b = []
		for c in a:
			#split text again based on blank spaces
			d = string.split(c)
			for e in d:
				f = ""
				for ch in e:
					#screen out unwanted special characters
					if ch not in ['<', "'", '"', '>']:
						#break up URLs into words using spaces, etc.
						if not ((ch in string.digits) or (ch in string.letters) or (ch in "%")):
							ch = ' '
						f = f + ch
				#add cleaned up word/phrase to new list--list contains clean attributes and their
				#values following
				f = string.strip(f)
				if len(f) > 0:
					b.append(f)

		#writeDebugInfo("b=" + `b` + "\n")
		#writeDebugInfo("tagWordEntry=" + `tagWordEntry` + "\n")

		#build domain text string by picking out attribute values we're interested in
		s = ""
		for i in range(0, len(b)-1):
			if string.upper(b[i]) in tagWordEntry:
				#s = s + " " + b[i+1]
				for g in string.split(b[i+1]):
					beginTagText[g] = None

		#writeDebugInfo("returning s = " + `s` + "\n")

		#beginTagText[s] = None
		return " "

def extraWhiteReplaceFunction(match):
	#always return 1 space/blank char
	return " "

def saveScriptStringsReplaceFunction(match):
	#process scripts--save all strings from them after converting non-word characters to space/blank;
	#throw everything else (besides strings) away; delimit by *ss* and *se* markers for ease in
	#spotting in database
	s = match.group("script")
	savedStrings = ""
	currentString = ""
	stringStack = []

	for ch in s:
		if ch == '"' or chr == "'":
			if stringStack and (ch == stringStack[-1]):
				stringStack = stringStack[:-1]
				if not stringStack:
					savedStrings = savedStrings + " " + currentString
					currentString = ""
			else:
				stringStack.append(ch)
		else:
			if stringStack:
				if ((ch in string.digits) or (ch in string.letters) or (ch in "%")):
					currentString = currentString + ch
				else:
					currentString = currentString + ' '

	return "\n*ss*\n" + savedStrings + "\n*se*\n"

def removeStopWordsAndPunctuation(s):
	list = string.split(s, " ")
	newStr = ""
	nonDigits = string.letters + "%"
	digits = string.digits + string.whitespace

	for item in list:
		item = string.strip(item)
		word = string.upper(item)
		if not stopwords.has_key(word):
			newItem = ""
			isWord = 0
			for c in item:
				if c in nonDigits:
					newItem = newItem + c
					isWord = 1
				else:
					if c in digits:
						newItem = newItem + c
					else:
						newItem = newItem + " "
			if isWord:
				newStr = newStr + newItem + " "
		#else:
			#writeDebugInfo("stopped word = " + item + "\n")
	return newStr

def stripHtml(s):
	global beginTagText

	#init to empty dictionary
	beginTagText = {}

	#The plan:
		#1) find zope stop words and strip them out of text
		#2) remove all punctuation chars in the text
	#Future
		#1) future: by stemming we can reduce size; stemming would be used on the text and on the
		#	query also
		#2)	future: handle %hh characters at some point
	#Ready to test:
		#1)	find scripts, and save all text in quotes from them (single and double)
		#2) put begin tag data at bottom of stripped page after running through a dictionary to remove
		#	duplicates
		#Did the following more or less:
			#1) get rid of everything matching <style>.*?</style>
			#2) find all <!--.*?--> and keep the text in between while getting rid of the comment brackets
			#3) find all <[^/]*?>
			#4) have dict of tags with value telling if they have useful/"domain" text in them (as attributes)
			#5) compare <[^/].*?> to tag types in this dictionary
			#6) if <[^/].*?> isn't a tag with useful "domain" text, then discard it
			#7) otherwise keep the useful text from it:
			#	find stuff after equal signs (and optional quotes) up till ending non-word character
			#8) if <[^/].*?> isn't in dictionary, log it (for debugging) and keep
			#	it--it might be part of some real text
			#9) find all </.*?> and throw them away
			#10) what is left should be good for indexing
	#Consider usefulness of
		#1) look for "title" attribute in everything; used for tool tips text. Would the text we
		#	 would get be that useful?
	#-------------------------------------------------------------------------------------------------

	#get rid of style instructions
	stylePat = re.compile(r'(?P<eliminate><style.*?</style.*?>)', re.IGNORECASE | re.DOTALL)
	s = stylePat.sub(eliminateReplaceFunction, s)

	#keep comments, but get rid of their beginning and ending tags; don't worry about comments
	#in a script because the whole script can be a comment
	commentPat = re.compile(r'<!--(?P<keep>.*?)-->', re.IGNORECASE | re.DOTALL)
	s = commentPat.sub(keepReplaceFunction, s)

	#process scripts--save all strings from them; throw everything else away; delimit by
	# *ss* and *se* markers for ease in spotting in database
	beginScriptPat = re.compile(r'<script>(?P<script>.+?)</script>', re.IGNORECASE | re.DOTALL)
	s = beginScriptPat.sub(saveScriptStringsReplaceFunction, s)

	#get rid of end tags--period!
	endTagPat = re.compile(r'(?P<eliminate></.*?>)', re.IGNORECASE | re.DOTALL)
	s = endTagPat.sub(eliminateReplaceFunction, s)

	#get rid of &nbsp;
	nbspPat = re.compile(r'&nbsp;', re.IGNORECASE | re.DOTALL)
	s = nbspPat.sub(eliminateReplaceFunction, s)

	#save beginning tag useful domain text in a dict (beginTagText) and add to end of text later
	beginTagPat = re.compile(r'(?P<examine><.+?>)', re.IGNORECASE | re.DOTALL)
	s = beginTagPat.sub(saveBeginTagTextReplaceFunction, s)

 	#add any beginning tag attribute words that were saved at the end; sort them to eliminate/reduce
 	#any false proximity searches
 	btt = " "
 	bttkeys = beginTagText.keys()
 	bttkeys.sort()
 	for k in bttkeys:
 		btt = btt + k + " "

	#mark them for now so we can spot them in the database more easily
	s = s + " " + btt + " "

	#get rid of stop words
	s = removeStopWordsAndPunctuation(s)

	#get rid of extra whitespace including blank lines
	extraWhitePat = re.compile(r'(?P<extraWhite>\s\s+)', re.IGNORECASE | re.DOTALL)
 	s = extraWhitePat.sub(extraWhiteReplaceFunction, s)

	#perform stemming on the words that are left
	listOfWords = string.split(s, " ")
 	listOfStemmedWords = Porter.stem(listOfWords)
 	s = ""
 	for w in listOfStemmedWords:
 		s = s + w + " "

	return s
