
ADZAPPER_CONFIG_FILE = 'adzapper.conf'

#standard Python imports--do non-standard ones after Python path is modified when the
#program is actually run. The startup .bat file will supply the path info.
import traceback
import socket
import select
import string
import os
import sys
import time

HOST = ''			#local
ZAP_PORT = 8579		#port for ad-blocking communications

def tracebackAsString(limit):
	type, value, tb = sys.exc_info()
	aList = traceback.format_exception(type, value, tb, limit)
	formattedString = ""
	for s in aList:
		formattedString = formattedString + s + "\n"
	return formattedString

def writeDebugInfo(msg):
	debugFile = open("c:\\debug.txt", "a+")
	debugFile.write(msg)
	debugFile.close()
	return

class combined_engine:
	# holder object for adzapper_engine and zaplet_engine
    def __init__(self):
        self.adzapper = None
        self.zaplet = None
        return

    def restart(self):
        self.zaplet.__init__(self.adzapper.zaplet_dirs,self.adzapper.site_zaplet_dir)
    	return

def makeCombinedEngine(configFileDirectory):
    if configFileDirectory[-1] != "\\":
    	configFileDirectory = configFileDirectory + "\\"

    adzapper_config_file = configFileDirectory + ADZAPPER_CONFIG_FILE

    # initialize the adzapper and zaplet engines
    # make the holder object for the combined adzapper and zaplet engines
    ce = combined_engine()

    # initialize adzapper engine - for adzapper defaults, ACLs, etc.
    ce.adzapper = adzapper_engine.adzapper_engine(adzapper_config_file)

    port = ce.adzapper.port

    # initialize zaplet engine - for blocking URLs
    #
    # zaplet_engine needs a list of dirs and the site zaplet dir
    ce.zaplet = zaplet_engine.zaplet_engine(ce.adzapper.zaplet_dirs,ce.adzapper.site_zaplet_dir)

    return ce

def serve(configFileDirectory):
	#create instance of adzapper engine
	ce = makeCombinedEngine(configFileDirectory)

	serversock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
	serversock.bind(HOST, ZAP_PORT)
	serversock.listen(5)

	print "Ad-blocker is ready for requests...\n"

	try:
		while 1:
			#give other programs some time to run
			time.sleep(0.1)

			#poll the socket for activity
			input, output, exc = select.select([serversock], [], [], 0)

			for sock in input:
				try:
					handlersock, addr = sock.accept()
				except Exception, data:
					print "Error accepting client socket: " + `data` + "\n"
					print tracebackAsString(25)
					break

				try:
					rcvdata = handlersock.recv(1024)
				except Exception, data:
					try:
						handlersock.close()
					except:
						pass
					print "Error receiving client data: " + `data` + "\n"
					print tracebackAsString(25)
					break

				try:
					argsList = string.split(rcvdata, "|")
					command = argsList[0]
					if command == "zapcheck":
						url = argsList[1]
						#print "url=" + `url` + "\n"
						[block, block_popups, filter_list] = ce.zaplet.urlCheckBlockAndFilter(url)
						if block != 0:
							handlersock.send("block")
							print "BLOCKED " + `url` + "\n"
						else:
							handlersock.send("don't block")
							#print "don't block\n"
						handlersock.close()
					else:
						#handle shutdown command eventually
						handlersock.send("?")
						handlersock.close()
				except Exception, data:
					print "Error processing client request: " + `data` + "\n"
					print tracebackAsString(25)
					try:
						handlersock.close()
					except:
						pass
					break

	except Exception, data:
		print "Error: " + `data`
		print tracebackAsString(25)
		try:
			serversock.close()
		except:
			pass

	return

if __name__ == "__main__":
	if len(sys.argv) != 3 or sys.argv[2][:2] != "-p":
		print
		print "Usage:"
		print "python zapAds.py <configFileDirectory> <-pPythonPath>"
		print
		print "Where PythonPath consists of Python path segments separated by semi-colons;"
		print "for example: -ph:\Website\Extensions;h:\Website\bin\lib;h:\Website\lib\python"
		print
		print "In the above example, Website represents the main Zope installation folder."
		print
	else:
		#fix Python path
		pythonPathInfo = sys.argv[2][2:]
		pathSegments = string.split(pythonPathInfo, ";")
		for s in pathSegments:
			sys.path.append(s)

		print
		print "Modified Python path=" + `sys.path`
		print

		#now that the Python path has been changed, import the non-standard stuff
		from adzapper import adzapper_engine
		from zaplet import zaplet_engine

		serve(sys.argv[1])
